#ifndef _K_MEAN_H_
#define _K_MEAN_H_
enum {
	K_MEAN_ERROR = -1,
	K_MEAN_SUCCESS,
};
/*
 * fields of struct k_mean_context
 * k : number of means
 * dim: dimension of data
 * weight : array of the weights k-th mean for viewed data
 * mean   : array of means
 * dist   : maximal Euclid distance in claster
 */
struct k_mean_context {
	int k, dim;
	double *weight, *mean, *dist;
};

struct k_mean_context * k_means_new();
void k_mean_init_(struct k_mean_context *ctx);
void k_mean_free(struct k_mean_context **ctx);
void k_mean_clear(struct k_mean_context *ctx);

/* Set number of mean. */
void k_mean_set_nmean(struct k_mean_context *ctx, int nmean);
/* Get number of mean. */
int k_mean_get_nmean(struct k_mean_context *ctx);

/* Set fitted data item dimension. */
void k_mean_set_dimension(struct k_mean_context *ctx, int dim);
/* Get fitted data item dimension. */
int k_mean_get_dimension(struct k_mean_context *ctx);

/* Set error value for end of iteration process. */
void k_mean_set_error(struct k_mean_context *ctx, double error);
/* Get error value for end of iteration process. */
double k_mean_get_error(struct k_mean_context *ctx);

/* Set n-th mean component weight. */
void k_mean_set_nth_weight(struct k_mean_context *ctx, int nth, double weight);
/* Get n-th mean component weight. */
double k_mean_get_nth_weight(struct k_mean_context *ctx, int nth);

/* Set n-th mean. */
void k_mean_set_nth_mean(struct k_mean_context *ctx, int nth, const double *mu);
/* Get n-th mean. */
void k_mean_get_nth_mean(const struct k_mean_context *ctx, int nth, double *mu);

/* Set n-th distance array. */
void k_mean_set_nth_distance(struct k_mean_context *ctx, int nth, const double *dist);
/* Get n-th distance array. */
void k_mean_get_nth_distance(const struct k_mean_context *ctx, int nth, double *cmx);

/*
 * Converge the k-means-algorithm, repeating k-means-steps until logarithm of the
 * likelihood function stops to change. Threshold of the change is given by epsilon.
 */
int k_mean_converge(struct k_mean_context *ctx, double *x, int nx, double epsilon);

/*
 * Determine if a sample x represents model or not.
 * return: K_MEAN_SUCCESS - if x represents model and data is valid
 *         K_MEAN_ERROR - else
 */
int k_mean_sample_in_model(struct k_mean_context *ctx, double *x);

/*
 * Update k_mean_context using new sample data
 */
void k_mean_update(struct k_mean_context *ctx, double *x);
#endif //_K_MEAN_H_
